=== modified file 'NEWS'
--- NEWS	2006-07-11 21:46:11 +0000
+++ NEWS	2006-07-12 03:16:07 +0000
@@ -83,6 +83,10 @@
      Also updates things like 'http+pycurl://' if pycurl is not present.
      (John Arbash Meinel) (Malone #47821, #52204)
 
+   * New env variable BZR_PROGRESS_BAR, sets the default progress bar type.
+     Can be set to 'none' or 'dummy' to disable the progress bar, 'dots' or 
+     'tty' to create the respective type. (John Arbash Meinel, #42197, #51107)
+
   BUG FIXES:
 
     * Fix shadowed definition of TestLocationConfig that caused some 

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-07-11 15:05:36 +0000
+++ bzrlib/errors.py	2006-07-12 02:20:47 +0000
@@ -970,6 +970,14 @@
     """A nested progress bar was not 'finished' correctly."""
 
 
+class InvalidProgressBarType(BzrNewError):
+    """Environment variable BZR_PROGRESS_BAR='%(bar_type)s is not a supported type
+Select one of: %(valid_types)s"""
+
+    def __init__(self, bar_type, valid_types):
+        BzrNewError.__init__(self, bar_type=bar_type, valid_types=valid_types)
+
+
 class UnsupportedOperation(BzrNewError):
     """The method %(mname)s is not supported on objects of type %(tname)s."""
     def __init__(self, method, method_self):

=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2006-06-21 02:33:40 +0000
+++ bzrlib/progress.py	2006-07-10 23:00:44 +0000
@@ -39,17 +39,16 @@
 import sys
 import time
 import os
-from collections import deque
-
 
 import bzrlib.errors as errors
-from bzrlib.trace import mutter 
+from bzrlib.trace import mutter
 
 
 def _supports_progress(f):
-    if not hasattr(f, 'isatty'):
+    isatty = getattr(f, 'isatty', None)
+    if isatty is None:
         return False
-    if not f.isatty():
+    if not isatty():
         return False
     if os.environ.get('TERM') == 'dumb':
         # e.g. emacs compile window
@@ -57,16 +56,31 @@
     return True
 
 
+_progress_bar_types = {}
+
 
 def ProgressBar(to_file=None, **kwargs):
     """Abstract factory"""
     if to_file is None:
         to_file = sys.stderr
-    if _supports_progress(to_file):
-        return TTYProgressBar(to_file=to_file, **kwargs)
+    requested_bar_type = os.environ.get('BZR_PROGRESS_BAR')
+    # An value of '' or not set reverts to standard processing
+    if requested_bar_type in (None, ''):
+        if _supports_progress(to_file):
+            return TTYProgressBar(to_file=to_file, **kwargs)
+        else:
+            return DotsProgressBar(to_file=to_file, **kwargs)
     else:
-        return DotsProgressBar(to_file=to_file, **kwargs)
-    
+        # Minor sanitation to prevent spurious errors
+        requested_bar_type = requested_bar_type.lower().strip()
+        # TODO: jam 20060710 Arguably we shouldn't raise an exception
+        #       but should instead just disable progress bars if we
+        #       don't recognize the type
+        if requested_bar_type not in _progress_bar_types:
+            raise errors.InvalidProgressBarType(requested_bar_type,
+                                                _progress_bar_types.keys())
+        return _progress_bar_types[requested_bar_type](to_file=to_file, **kwargs)
+
  
 class ProgressBarStack(object):
     """A stack of progress bars."""
@@ -93,7 +107,7 @@
         self._show_count = show_count
         self._to_messages_file = to_messages_file
         self._stack = []
-        self._klass = klass or TTYProgressBar
+        self._klass = klass or ProgressBar
 
     def top(self):
         if len(self._stack) != 0:
@@ -206,6 +220,10 @@
         return DummyProgress(**kwargs)
 
 
+_progress_bar_types['dummy'] = DummyProgress
+_progress_bar_types['none'] = DummyProgress
+
+
 class DotsProgressBar(_BaseProgressBar):
 
     def __init__(self, **kwargs):
@@ -233,6 +251,9 @@
     def child_update(self, message, current, total):
         self.tick()
 
+
+_progress_bar_types['dots'] = DotsProgressBar
+
     
 class TTYProgressBar(_BaseProgressBar):
     """Progress bar display object.
@@ -262,8 +283,10 @@
         _BaseProgressBar.__init__(self, **kwargs)
         self.spin_pos = 0
         self.width = terminal_width()
-        self.last_updates = deque()
+        self.last_updates = []
+        self._max_last_updates = 10
         self.child_fraction = 0
+        self._have_output = False
     
 
     def throttle(self, old_msg):
@@ -284,6 +307,8 @@
             return True
 
         self.last_updates.append(now - self.last_update)
+        # Don't let the queue grow without bound
+        self.last_updates = self.last_updates[-self._max_last_updates:]
         self.last_update = now
         return False
         
@@ -405,13 +430,19 @@
 
         assert len(m) < self.width
         self.to_file.write('\r' + m.ljust(self.width - 1))
+        self._have_output = True
         #self.to_file.flush()
             
     def clear(self):        
-        self.to_file.write('\r%s\r' % (' ' * (self.width - 1)))
+        if self._have_output:
+            self.to_file.write('\r%s\r' % (' ' * (self.width - 1)))
+        self._have_output = False
         #self.to_file.flush()        
 
 
+_progress_bar_types['tty'] = TTYProgressBar
+
+
 class ChildProgress(_BaseProgressBar):
     """A progress indicator that pushes its data to the parent"""
 
@@ -487,8 +518,6 @@
     assert total_duration >= elapsed
 
     if last_updates and len(last_updates) >= n_recent:
-        while len(last_updates) > n_recent:
-            last_updates.popleft()
         avg = sum(last_updates) / float(len(last_updates))
         time_left = avg * (total - current)
 

=== modified file 'bzrlib/tests/test_progress.py'
--- bzrlib/tests/test_progress.py	2006-06-21 03:14:59 +0000
+++ bzrlib/tests/test_progress.py	2006-07-10 23:00:44 +0000
@@ -14,8 +14,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import os
 from StringIO import StringIO
 
+from bzrlib import errors
 from bzrlib.progress import (
         DummyProgress, ChildProgress,
         TTYProgressBar,
@@ -24,6 +26,7 @@
         )
 from bzrlib.tests import TestCase
 
+
 class FakeStack:
     def __init__(self, top):
         self.__top = top
@@ -44,6 +47,20 @@
             self.always_throttled = False
         
 
+class _TTYStringIO(StringIO):
+    """A helper class which makes a StringIO look like a terminal"""
+
+    def isatty(self):
+        return True
+
+
+class _NonTTYStringIO(StringIO):
+    """Helper that implements isatty() but returns False"""
+
+    def isatty(self):
+        return False
+
+
 class TestProgress(TestCase):
     def setUp(self):
         q = DummyProgress()
@@ -120,3 +137,174 @@
         # Updates after a second should not be squelched
         pb.update('me', 1, 1)
         self.assertFalse(pb.always_throttled)
+
+    def test_clear(self):
+        sio = StringIO()
+        pb = TTYProgressBar(to_file=sio, show_eta=False)
+        pb.width = 20 # Just make it easier to test
+        # This should not output anything
+        pb.clear()
+        # These two should not be displayed because
+        # of throttling
+        pb.update('foo', 1, 3)
+        pb.update('bar', 2, 3)
+        # So pb.clear() has nothing to do
+        pb.clear()
+
+        # Make sure the next update isn't throttled
+        pb.start_time -= 1
+        pb.update('baz', 3, 3)
+        pb.clear()
+
+        self.assertEqual('\r[=========] baz 3/3'
+                         '\r                   \r',
+                         sio.getvalue())
+
+    def test_no_eta(self):
+        # An old version of the progress bar would
+        # store every update if show_eta was false
+        # because the eta routine was where it was
+        # cleaned out
+        pb = InstrumentedProgress(to_file=StringIO(), show_eta=False)
+        # Just make sure this first few are throttled
+        pb.start_time += 5
+
+        # These messages are throttled, and don't contribute
+        for count in xrange(100):
+            pb.update('x', count, 300)
+        self.assertEqual(0, len(pb.last_updates))
+
+        # Unthrottle by time
+        pb.start_time -= 10
+
+        # These happen too fast, so only one gets through
+        for count in xrange(100):
+            pb.update('x', count+100, 200)
+        self.assertEqual(1, len(pb.last_updates))
+
+        pb.MIN_PAUSE = 0.0
+
+        # But all of these go through, don't let the
+        # last_update list grow without bound
+        for count in xrange(100):
+            pb.update('x', count+100, 200)
+
+        self.assertEqual(pb._max_last_updates, len(pb.last_updates))
+
+
+class TestProgressTypes(TestCase):
+    """Test that the right ProgressBar gets instantiated at the right time."""
+
+    def get_nested(self, outf, term, env_progress=None):
+        """Setup so that ProgressBar thinks we are in the supplied terminal."""
+        orig_term = os.environ.get('TERM')
+        orig_progress = os.environ.get('BZR_PROGRESS_BAR')
+        os.environ['TERM'] = term
+        if env_progress is not None:
+            os.environ['BZR_PROGRESS_BAR'] = env_progress
+        elif orig_progress is not None:
+            del os.environ['BZR_PROGRESS_BAR']
+
+        def reset():
+            if orig_term is None:
+                del os.environ['TERM']
+            else:
+                os.environ['TERM'] = orig_term
+            # We may have never created BZR_PROGRESS_BAR
+            # So we can't just delete like we can 'TERM' (which is always set)
+            if orig_progress is None:
+                if 'BZR_PROGRESS_BAR' in os.environ:
+                    del os.environ['BZR_PROGRESS_BAR']
+            else:
+                os.environ['BZR_PROGRESS_BAR'] = orig_progress
+
+        self.addCleanup(reset)
+
+        stack = ProgressBarStack(to_file=outf)
+        pb = stack.get_nested()
+        pb.start_time -= 1 # Make sure it is ready to write
+        pb.width = 20 # And it is of reasonable size
+        return pb
+
+    def test_tty_progress(self):
+        # Make sure the ProgressBarStack thinks it is
+        # writing out to a terminal, and thus uses a TTYProgressBar
+        out = _TTYStringIO()
+        pb = self.get_nested(out, 'xterm')
+        self.assertIsInstance(pb, TTYProgressBar)
+        try:
+            pb.update('foo', 1, 2)
+            pb.update('bar', 2, 2)
+        finally:
+            pb.finished()
+
+        self.assertEqual('\r/ [====   ] foo 1/2'
+                         '\r- [=======] bar 2/2'
+                         '\r                   \r',
+                         out.getvalue())
+
+    def test_dots_progress(self):
+        # Make sure the ProgressBarStack thinks it is
+        # not writing out to a terminal, and thus uses a 
+        # DotsProgressBar
+        out = _NonTTYStringIO()
+        pb = self.get_nested(out, 'xterm')
+        self.assertIsInstance(pb, DotsProgressBar)
+        try:
+            pb.update('foo', 1, 2)
+            pb.update('bar', 2, 2)
+        finally:
+            pb.finished()
+
+        self.assertEqual('foo: .'
+                         '\nbar: .'
+                         '\n',
+                         out.getvalue())
+
+    def test_no_isatty_progress(self):
+        # Make sure ProgressBarStack handles a plain StringIO()
+        import cStringIO
+        out = cStringIO.StringIO()
+        pb = self.get_nested(out, 'xterm')
+        pb.finished()
+        self.assertIsInstance(pb, DotsProgressBar)
+
+    def test_dumb_progress(self):
+        # Make sure the ProgressBarStack thinks it is writing out to a 
+        # terminal, but it is the emacs 'dumb' terminal, so it uses
+        # Dots
+        out = _TTYStringIO()
+        pb = self.get_nested(out, 'dumb')
+        pb.finished()
+        self.assertIsInstance(pb, DotsProgressBar)
+
+    def test_progress_env_tty(self):
+        # The environ variable BZR_PROGRESS_BAR controls what type of
+        # progress bar we will get, even if it wouldn't usually be that type
+        import cStringIO
+
+        # Usually, this would be a DotsProgressBar
+        out = cStringIO.StringIO()
+        pb = self.get_nested(out, 'dumb', 'tty')
+        pb.finished()
+        # Even though we are not a tty, the env_var will override
+        self.assertIsInstance(pb, TTYProgressBar)
+
+    def test_progress_env_dots(self):
+        # Even though we are in a tty, the env_var will override
+        out = _TTYStringIO()
+        pb = self.get_nested(out, 'xterm', 'dots')
+        pb.finished()
+        self.assertIsInstance(pb, DotsProgressBar)
+
+    def test_progress_env_none(self):
+        # Even though we are in a valid tty, no progress
+        out = _TTYStringIO()
+        pb = self.get_nested(out, 'xterm', 'none')
+        pb.finished()
+        self.assertIsInstance(pb, DummyProgress)
+
+    def test_progress_env_invalid(self):
+        out = _TTYStringIO()
+        self.assertRaises(errors.InvalidProgressBarType, self.get_nested,
+            out, 'xterm', 'nonexistant')

=== modified file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	2006-06-21 02:33:40 +0000
+++ bzrlib/tests/test_ui.py	2006-07-12 02:45:22 +0000
@@ -26,6 +26,7 @@
 import bzrlib.errors as errors
 from bzrlib.progress import TTYProgressBar, ProgressBarStack
 from bzrlib.tests import TestCase
+from bzrlib.tests.test_progress import _TTYStringIO
 from bzrlib.ui import SilentUIFactory
 from bzrlib.ui.text import TextUIFactory
 
@@ -63,18 +64,41 @@
     def test_progress_note(self):
         stderr = StringIO()
         stdout = StringIO()
-        ui_factory = TextUIFactory()
-        pb = ui_factory.nested_progress_bar()
-        try:
-            pb.to_messages_file = stdout
-            ui_factory._progress_bar_stack.bottom().to_file = stderr
+        ui_factory = TextUIFactory(bar_type=TTYProgressBar)
+        pb = ui_factory.nested_progress_bar()
+        try:
+            pb.to_messages_file = stdout
+            ui_factory._progress_bar_stack.bottom().to_file = stderr
+            result = pb.note('t')
+            self.assertEqual(None, result)
+            self.assertEqual("t\n", stdout.getvalue())
+            # Since there was no update() call, there should be no clear() call
+            self.failIf(re.search(r'^\r {10,}\r$', stderr.getvalue()) is not None,
+                        'We cleared the stderr without anything to put there')
+        finally:
+            pb.finished()
+
+    def test_progress_note_clears(self):
+        stderr = StringIO()
+        stdout = StringIO()
+        # The PQM redirects the output to a file, so it
+        # defaults to creating a Dots progress bar. we
+        # need to force it to believe we are a TTY
+        ui_factory = TextUIFactory(bar_type=TTYProgressBar)
+        pb = ui_factory.nested_progress_bar()
+        try:
+            pb.to_messages_file = stdout
+            ui_factory._progress_bar_stack.bottom().to_file = stderr
+            # Create a progress update that isn't throttled
+            pb.start_time -= 10
+            pb.update('x', 1, 1)
             result = pb.note('t')
             self.assertEqual(None, result)
             self.assertEqual("t\n", stdout.getvalue())
             # the exact contents will depend on the terminal width and we don't
             # care about that right now - but you're probably running it on at
             # least a 10-character wide terminal :)
-            self.assertContainsRe(stderr.getvalue(), r'^\r {10,}\r$')
+            self.assertContainsRe(stderr.getvalue(), r'\r {10,}\r$')
         finally:
             pb.finished()
 
@@ -153,7 +177,7 @@
     def test_text_factory_prompts_and_clears(self):
         # a get_boolean call should clear the pb before prompting
         factory = bzrlib.ui.text.TextUIFactory()
-        factory.stdout = StringIO()
+        factory.stdout = _TTYStringIO()
         factory.stdin = StringIO("yada\ny\n")
         pb = self.apply_redirected(
             factory.stdin, factory.stdout, factory.stdout, factory.nested_progress_bar)
@@ -174,4 +198,4 @@
             "\r   *" 
             "\rwhat do you want\\? \\[y/n\\]:what do you want\\? \\[y/n\\]:", 
             output):
-            self.fail("didn't match factory output %r" % factory)
+            self.fail("didn't match factory output %r, %s" % (factory, output))

